import numpy as np
import pandas as pd
import os
import yaml
from qa4sm_preprocessing.level2.smos import SMOSL2Reader
from smos.misc import read_summary_yml, get_first_last_day_images
from datetime import datetime

_default_variables = (
    "Soil_Moisture",
    "Science_Flags",
    "Confidence_Flags",
    "Chi_2_P",
    "RFI_Prob",
    "N_RFI_X",
    "N_RFI_Y",
    "M_AVA0",
)

def swath2ts(img_path, ts_path, variables=_default_variables,
             startdate=None, enddate=None, memory=4):
    """
    Convert SMOS L2 swath data to time series in IndexedRaggedTs format.

    Parameters
    ----------
    img_path: str
        Local (root) directory where the annual folder containing SMOS L2 SM
        swath data are found.
    ts_path: str
        Local directory where the converted time series data will be stored.
    variables: tuple or str, optional (default: None)
        List of variables to include, None will use the default variables
        "Soil_Moisture",
        "Soil_Moisture_DQX",
        "Science_Flags",
        "Confidence_Flags",
        "Processing_Flags",
        "Chi_2_P",
        "RFI_Prob",
        "N_RFI_X",
        "N_RFI_Y",
        "M_AVA0",
        "acquisition_time"
    startdate: str or datetime, optional (default: None)
        First day of the available swath data that should be included in the
        time series. If None is passed, then the first available day is used.
    enddate: str or datetime, optional (default: None)
        Last day of the available swath data that should be included in the
        time series. If None is passed, then the last available day is used.
    memory : float, optional (default: 4)
        Size of available memory in GB. More memory will lead to a faster
        conversion.
    """
    variables = [v for v in np.atleast_1d(variables)]

    if "acquisition_time" not in variables:
        variables.append("acquisition_time")

    reader = SMOSL2Reader(img_path, varnames=variables,
                          add_overpass_flag=True)

    first_day, last_day = get_first_last_day_images(img_path)

    start = pd.to_datetime(startdate).to_pydatetime() if startdate is not None else first_day
    end = pd.to_datetime(enddate).to_pydatetime() if enddate is not None else last_day

    if start is None or end is None:
        raise ValueError("No start and/or end date provided.")

    out_file = os.path.join(ts_path, f"overview.yml")

    if os.path.isfile(out_file):
        props = read_summary_yml(ts_path)
        if start < pd.to_datetime(props['last_day']).to_pydatetime():
            raise ValueError("Cannot prepend data to time series, or replace "
                             "existing values. Choose different start date.")

    props = {'comment': "DO NOT CHANGE THIS FILE MANUALLY! Required for data update.",
             'last_day': str(end), 'last_update': str(datetime.now()),
             'parameters': [str(v) for v in reader.varnames]}

    r = reader.repurpose(
        outpath=ts_path,
        start=start,
        end=end,
        memory=memory,
        overwrite=False,
        imgbaseconnection=True,
    )

    if r is not None:
        with open(out_file, 'w') as f:
            yaml.dump(props, f, default_flow_style=False, sort_keys=False)

def extend_ts(img_path, ts_path, memory=4):
    """
    Append new image data to an existing time series record.
    This will use the last_day from summary.yml in the time series
    directory to decide which date the update should start from and
    the available image directories to decide how many images can be
    appended.

    Parameters
    ----------
    img_path: str
        Path where the annual folders containing downloaded SMOS L2 images
        are stored
    ts_path: str
        Path where the converted time series (initially created using the
        reshuffle / swath2ts command) are stored.
    memory: int, optional (default: 4)
        Available memory in GB
    """
    out_file = os.path.join(ts_path, f"overview.yml")
    if not os.path.isfile(out_file):
        raise ValueError("No overview.yml found in the time series directory."
                         "Please use reshuffle / swath2ts for initial time "
                         f"series setup or provide overview.yml in {ts_path}.")

    props = read_summary_yml(ts_path)
    startdate = pd.to_datetime(props['last_day']).to_pydatetime()
    _, last_day = get_first_last_day_images(img_path)

    if startdate is None or last_day is None:
        raise ValueError("No start and/or end date provided.")

    startdate = pd.to_datetime(startdate).to_pydatetime()
    last_day = pd.to_datetime(last_day).to_pydatetime()

    if startdate < last_day:

        reader = SMOSL2Reader(img_path)

        print(f"Extent TimeSeries data From: {startdate}, To: {last_day}")

        r = reader.repurpose(
            outpath=ts_path,
            start=startdate,
            end=last_day,
            memory=memory,
            imgbaseconnection=True,
            overwrite=False,
            append=True,
        )

        if r is not None:
            props['last_day'] = str(last_day)
            props['last_update'] = str(datetime.now())

            with open(out_file, 'w') as f:
                yaml.dump(props, f, default_flow_style=False, sort_keys=False)

    else:
        print(f"No extension required From: {startdate} To: {last_day}")



# -*- coding: utf-8 -*-
# The MIT License (MIT)
#
# Copyright (c) 2021, TU Wien
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import argparse
from datetime import datetime
from netCDF4 import Dataset


def firstfile(input_root):
    '''
    Find the first file and read lat and lon and all variables

    Parameters
    --------
    input_root : str
        Path to the root dir of image files

    Returns
    --------
    firstfile : str
        Path to the first found image file in the input_root
    vars : list
        List of variables in the file
    '''

    for curr, subdirs, files in os.walk(input_root):
        for f in files:
            if len(f) > 0:
                firstfile = os.path.join(input_root, curr, f)
                ds = Dataset(firstfile)
                vars = ds.variables.keys()

                return firstfile, vars

    raise (IOError, 'No netcdf data found under the passed root directory')


def mkdate(datestring):
    """
    Create date string.
    Parameters
    ----------
    datestring : str
        Date string.
    Returns
    -------
    datestr : datetime
        Date string as datetime.
    """
    if len(datestring) == 10:
        return datetime.strptime(datestring, '%Y-%m-%d')
    if len(datestring) == 16:
        return datetime.strptime(datestring, '%Y-%m-%dT%H:%M')


def str2bool(val):
    if val in ['True', 'true', 't', 'T', '1']:
        return True
    else:
        return False


def parse_args(args):
    """
    Parse command line parameters for recursive download.

    Parameters
    ----------
    args : list of str
        Command line parameters as list of strings.
    Returns
    -------
    args : argparse.Namespace
        Command line arguments.
    """

    parser = argparse.ArgumentParser(
        description="Convert SMOS image data to time series format.")
    parser.add_argument("dataset_root",
                        help="Root of local filesystem where the "
                             "data is stored.")

    parser.add_argument("timeseries_root",
                        help="Root of local filesystem where the timeseries "
                             "should be stored.")

    parser.add_argument("start", type=mkdate,
                        help=("Startdate. Either in format YYYY-MM-DD or "
                              "YYYY-MM-DDTHH:MM."))

    parser.add_argument("end", type=mkdate,
                        help=("Enddate. Either in format YYYY-MM-DD or "
                              "YYYY-MM-DDTHH:MM."))

    parser.add_argument("--parameters", metavar="parameters", default=None,
                        nargs="+",
                        help=("Parameters to reshuffle into time series format. "
                              "e.g. Soil_Moisture. If this is not specified, all "
                              "variables in the image file will be reshuffled. "
                              "Default: None"))

    parser.add_argument("--only_good", type=str2bool, default='False',
                        help=("Use only observations with Quality_Flag 0 (GOOD), "
                              "if this is False, also 1-flagged (not recommended) "
                              "ones will be read and reshuffled, 2-flagged (missing)"
                              "values are always masked. Default: False"))

    parser.add_argument("--bbox", type=float, default=None, nargs=4,
                        help=("min_lon min_lat max_lon max_lat. "
                              "Bounding Box (lower left and upper right corner) "
                              "of subset area of global images to reshuffle (WGS84). "
                              "Default: None"))

    parser.add_argument("--imgbuffer", type=int, default=100,
                        help=("How many images to read at once. Bigger "
                              "numbers make the conversion faster but "
                              "consume more memory. Default: 100."))

    args = parser.parse_args(args)

    print(f"Converting SMOS IC data from {args.dataset_root} between "
          f"{args.start.isoformat()} and {args.end.isoformat()} "
          f"into folder {args.timeseries_root}. ")
    print(f"Masking to include only recommended values is "
          f"{'activated' if args.only_good else 'deactivated'}.")
    if args.bbox is not None:
        print(f"Bounding Box used: {str(args.bbox)}")

    return args



"""
Module to synchronize SMOS L2 data from FTP to local disk
"""
import os
from pathlib import Path
import subprocess
from datetime import datetime
from tqdm import tqdm
import pandas as pd
from calendar import monthrange
from pathlib import PurePosixPath
from smos.misc import get_first_last_day_images
import yaml

L2_START_DATE = datetime(2010, 6, 1)

def load_dotrc(path=None) -> dict:
    """
    Read FTP login credentials from .smosrc file.

    Parameters
    ----------
    path: str, optional (default: None)
        Path to the dotrc file. None will look in the default branch
        with is the home folder.

    Returns
    -------
    config: dict
        Elements from the dotrc file
    """
    if path is None:
        path = os.path.join(str(Path.home()), '.smosapirc')
    if not os.path.exists(path):
        raise ValueError(f'.smosapirc file not found at {path}. '
                         f'Create an account at https://eoiam-idp.eo.esa.int')
    config = {}
    with open(path) as f:
        for line in f.readlines():
            if "=" in line:
                k, v = line.strip().split("=", 1)
                if k in ("DISSEO_USERNAME", "DISSEO_PASSWORD"):
                    config[k] = v.strip(f""" "'""""")
    return config


class SmosDissEoFtp:
    def __init__(self, local_root, username=None, password=None, dotrc=None,
                 skip_lftp_verify=False):
        """
        Access to SMOS L2 data from FTP.

        Parameters
        ----------
        local_root: str
            Local root folder where the data from the FTP server is transferred
            into.
        username: str, optional (default: None)
            Username of your EO Sign In account. If None is passed here
            it will be derived from the .smosapirc file in the home directory.
            Create an account at https://eoiam-idp.eo.esa.int
        password: str, optional (default: None)
            Password for the EO Sign in account. If None is passed here
            it will be derived from the .smosapirc file in the home directory.
            Create an account at https://eoiam-idp.eo.esa.int
        dotrc: str, optional (default: None)
            Path to the .smosapirc file containing the FTP username and password.
            If None, then the file is assumed to be at $HOME/.smosapirc
                DISSEO_USERNAME=xxxx
                DISSEO_PASSWORD=xxxx
            Create an account at https://eoiam-idp.eo.esa.int
        skip_lftp_verify: bool, optional (default: False)
            Skip checking if lftp is available (for testing).
        """
        self.host = "ftps://smos-diss.eo.esa.int"
        self.ftp_root = PurePosixPath("/", "SMOS", "L2SM", "MIR_SMUDP2_nc")

        self.username = username
        self.password = password

        self.local_root = Path(local_root)
        os.makedirs(self.local_root, exist_ok=True)

        if self.username is None or self.password is None:
            config = load_dotrc(dotrc)
            if self.username is None:
                self.username = config['DISSEO_USERNAME']
            if self.password is None:
                self.password = config['DISSEO_PASSWORD']

        if not skip_lftp_verify:
            self.verify_lftp_installed()

    def verify_lftp_installed(self):
        """
        Call lftp command to check if program is available.
        Otherwise it has to be installed e.g. via apt-get install.
        """
        r = subprocess.run(["lftp", "--version"])

        if r.returncode != 0:
            raise ValueError("lftp command is not available. "
                             "Please install lftp: https://lftp.yar.ru/")

    def exec(self, cmd):
        cmd = [
            "lftp", "-c",
            f"open {self.host} && set ssl:verify-certificate no && "
            f"user {self.username} {self.password} && "
            f"{cmd} && "
            f"quit"
        ]

        r = subprocess.run(cmd, capture_output=True)

        return r

    def list(self, subpath='', filter='all'):
        """
        Create a list of all files and subdirectories under the passed
        path on the server.
        Directories end with /, files should have a file extension.

        Parameters
        ----------
        subpath: str, optional (default: '')
            Subdirectory on the server to look into.
            e.g. '/2020/01/01'
        filter: str, optional (default: 'all')
            - all: returns fils and folders
            - file: returns only files
            - dir: returns only directories

        Returns
        -------
        elements: list
            List of all files and/or folders under the subpath on the server
        """
        path = self.ftp_root
        if subpath not in [None, '']:
            path = path / PurePosixPath(subpath)
        cmd = f"cls {path}"
        r = self.exec(cmd)
        lst = r.stdout.decode("utf-8").splitlines()

        data = []
        for l in lst:
            d = l.split('/')[-1]
            if d == '':
                d = l.split('/')[-2]+'/'
            if d.endswith('/') and (filter in ['dir', 'all']):
                data.append(d)
            if not d.endswith('/') and (filter in ['file', 'all']):
                data.append(d)

        return data

    def last_available_day(self):
        """
        Get the latest available day on the server (incomplete directory).
        We want to exclude this day from downloading.

        Returns
        -------
        last_date:

        """
        last_year = [int(y.replace('/', '')) for y in self.list(filter='dir')][-1]
        last_month = [int(m.replace('/', '')) for m in self.list(subpath=str(last_year), filter='dir')][-1]
        last_day = [int(d.replace('/', '')) for d in self.list(subpath=f"{last_year}/{last_month:02}", filter='dir')][-1]

        return datetime(last_year, last_month, last_day)

    def list_all_available_days(self, date_from=L2_START_DATE,
                                date_to=datetime.now(), progressbar=True):
        """
        Shortcut to get a list of all available days (i.e. folders) on the
        server within the selected time frame.

        Parameters
        ----------
        date_from: str or datetime, optional
            First date of the time frame to check available days for on server.
            By default, we use the first date of SMOS L2 (2010-06-01)
        date_to: str or datetime, optional
            Last date of the time frame to check available days for on server.
            By default, we use the current date.
        progressbar: bool, optional (default: True)
            This operation will send some request to the server and may take
            some time. (De)activate a visual progress representation.

        Returns
        -------
        dates: list
            List of dates for which a folder exists on the server
        """
        date_to = pd.to_datetime(date_to).to_pydatetime()
        date_from = pd.to_datetime(date_from).to_pydatetime()

        dates = []
        years = [int(y.replace('/', '')) for y in self.list(filter='dir')]
        years = [y for y in years if ((y >= date_from.year) and (y <= date_to.year))]

        for year in tqdm(years, disable=not progressbar, description="Scanning FTP folder"):
            months = [int(m.replace('/', '')) for m in self.list(subpath=str(year), filter='dir')]
            if year == date_from.year:
                months = [m for m in months if m >= date_from.month]
            if year == date_to.year:
                months = [m for m in months if m <= date_to.month]
            for month in months:
                days = self.list(subpath=f"{year}/{month:02}", filter='dir')
                for day in days:
                    dt = datetime(int(year),
                                  int(month),
                                  int(day.replace('/', '')))
                    if date_from <= dt <= date_to:
                        dates.append(dt)

        return dates

    def sync(self, year, month, day=None, opts='', dry_run=False):
        """
        Download data from remote to local folder for a certain day.

        Parameters
        ----------
        year: int
            Year part of the date to download
        month: int
            Month part of the date to download
        day: int, optional
            Day part of the date to download. If not set, then
            the whole month is synced.
        opts: str, optional, default: ''
            Additional options that are added to the command
            mirror OPTS root_dir target_dir
            For all options see https://lftp.yar.ru/lftp-man.html
        dry_run: bool, optional, default=False
            Dry run does not actually download anything.
            Instead of the return value, the full command is returned

        Returns
        -------
        ret: str
            Return value or command (if dry_run)
        """
        _d = datetime(year, month, day if day is not None else monthrange(year, month)[1])
        if _d < L2_START_DATE:
            raise ValueError(f"Chosen date must be after {L2_START_DATE}")

        subpath = Path(str(year), f"{month:02}")

        if day is not None:
            subpath = subpath / f"{day:02}"

        target_path = self.local_root / subpath

        cmd = ["mirror -c"]
        if len(opts) > 0:
            cmd.append(opts)
        cmd.append(str(self.ftp_root / PurePosixPath(subpath)))
        cmd.append(f"{target_path}")
        cmd.append('−−no−perms')
        cmd = ' '.join(cmd)

        if dry_run:
            return cmd
        else:
            return self.exec(cmd)

    def sync_period(self, startdate, enddate, dry_run=False):
        """
        Synchronize SMOS L2 data between local root and FTP folder for days
        in the passed time frame. The last day on the server is usually not yet
        complete (i.e. swath files are missing). This will NOT be synchronized.

        Parameters
        ----------
        startdate: str or datetime
            First day to download data for (if available)
        enddate: str or datetime
            Last day to download data for (if available)

        Returns:
        -------
        ret: list
            List of return values or commands (if dry_run was chosen)
        """
        startdate = pd.to_datetime(startdate)
        enddate = pd.to_datetime(enddate)

        df = pd.Series(index=pd.date_range(startdate, enddate, freq='D'),
                       data=1)

        ret = []

        for year, ys in df.groupby(df.index.year):
            for month, ms in ys.groupby(ys.index.month):
                if len(ms) == monthrange(year, month)[1]:  # complete month (fast)
                    r = self.sync(int(year), int(month), day=None, dry_run=dry_run)
                    ret.append(r)
                else:  # individual days (slow)
                    for dt in ms.index.values:
                        dt = pd.Timestamp(dt).to_pydatetime()
                        r = self.sync(dt.year, dt.month, dt.day, dry_run=dry_run)
                        ret.append(r)

        first_day, last_day = get_first_last_day_images(str(self.local_root))

        props = dict(comment="DO NOT CHANGE THIS FILE MANUALLY! Required for data update.",
                     first_day=str(first_day) if first_day is not None else None,
                     last_day=str(last_day) if last_day is not None else None,
                     last_update=str(datetime.now()))

        with open(os.path.join(self.local_root, 'overview.yml'), 'w') as f:
            yaml.dump(props, f, default_flow_style=False, sort_keys=False)

        return ret



import click
from datetime import datetime, timedelta
import pandas as pd

from smos.smos_l2.download import SmosDissEoFtp, L2_START_DATE
from smos.smos_l2.reshuffle import swath2ts, extend_ts, _default_variables
from smos.misc import get_first_last_day_images

@click.command(
    "download",
    context_settings={'show_default': True},
    short_help="Download SMOS L2 data from FTP. This requires that the `lftp` "
               "program is installed on your system: https://lftp.yar.ru/")
@click.argument("path", type=click.Path(writable=True))
@click.option(
    '--startdate', '-s',
    type=click.STRING,
    default=str(L2_START_DATE.date()),
    help="Startdate in format YYYY-MM-DD. If not given, "
         "then the first available date of the product is used.")
@click.option(
    '--enddate', '-e',
    type=click.STRING,
    default=None,
    help="Enddate in format YYYY-MM-DD. If not given, "
         "then the last full day on the server is used.")
@click.option(
    "--username",
    type=click.STRING,
    default=None,
    help="The username you use to login at the FTP server. "
         "Required if no .smosapirc file exists in your home directory. "
         "Please create an account at https://eoiam-idp.eo.esa.int")
@click.option(
    "--password",
    type=click.STRING,
    default=None,
    help="The password you use to login at the FTP server. "
         "Required if no .smosapirc file exists in your home directory. "
         "Please create an account at https://eoiam-idp.eo.esa.int")
def cli_download(path,
                 startdate,
                 enddate,
                 username,
                 password,
    ):
    """
    Download SMOS L2 data within a chosen period. NOTE: Before using this
    program, create an account at https://eoiam-idp.eo.esa.int and ideally
    store you credentials in the file $HOME/.smosapirc (to avoid passing them
    as plain text).

    \b
    Required Parameters
    -------------------
    PATH: string (required)
        Path where the downloaded images are stored.
    """
    # The docstring above is slightly different to the normal python one to
    # display it properly on the command line.

    ftp = SmosDissEoFtp(path, username=username, password=password)

    if enddate is None:
        enddate = ftp.last_available_day() - timedelta(days=1)

    ftp.sync_period(startdate=pd.to_datetime(startdate).to_pydatetime(),
                    enddate=enddate)

@click.command(
    "update_img",
    context_settings={'show_default': True},
    short_help="Extend an existing record by downloading new files. "
               "This requires that the `lftp` program is installed on your "
               "system: https://lftp.yar.ru/")
@click.argument("path",
                type=click.Path(writable=True))
@click.option(
    "--username",
    type=click.STRING,
    default=None,
    help="The username you use to login at the FTP server. "
         "Required if no .smosapirc file exists in your home directory. "
         "Please create an account at https://eoiam-idp.eo.esa.int")
@click.option(
    "--password",
    type=click.STRING,
    default=None,
    help="The password you use to login at the FTP server. "
         "Required if no .smosapirc file exists in your home directory. "
         "Please create an account at https://eoiam-idp.eo.esa.int")
def cli_update_img(path,
                   username,
                   password):
    """
    Extend a locally existing SMOS L2 by downloading new files that
    don't yet exist locally. The last day on the server is usually incomplete
    and therefore ignored.
    NOTE: Before using this program, create an account at
    https://eoiam-idp.eo.esa.int and ideally store you credentials in the
    file $HOME/.smosapirc (to avoid passing them as plain text).
    NOTE: Use the `smos_l2 download` program first do create a local record
    to update with this function.

    \b
    Required Parameters
    -------------------
    PATH: string
        Path where previously downloaded SMOS L2 images are stored.
    """
    # The docstring above is slightly different to the normal python one to
    # display it properly on the command line.

    ftp = SmosDissEoFtp(path, username=username, password=password)
    enddate = ftp.last_available_day() - timedelta(days=1)
    # in case there are any incomplete days
    ftp.sync_period(startdate=get_first_last_day_images(path)[1],
                    enddate=str(enddate.date()))


@click.command(
    "reshuffle",
    context_settings={'show_default': True},
    short_help="Convert SMOS L2 swath images into time series.")
@click.argument("img_path", type=click.Path(readable=True))
@click.argument("ts_path", type=click.Path(writable=True))
@click.option(
    '--startdate',
    '-s',
    type=click.STRING,
    default=None,
    help="Format YYYY-MM-DD | First day to include in the"
    "time series. [default: Date of the first available image]")
@click.option(
    '--enddate',
    '-e',
    type=click.STRING,
    default=None,
    help="Format YYYY-MM-DD | Last day to include in the"
    "time series. [default: Date of the last available image]")
@click.option(
    '--variables',
    '-v',
    type=click.STRING,
    default=','.join(_default_variables),
    help="List of variables in the swath files to reshuffle. Multiple variables"
         " must be comma-separated.")
@click.option(
    '--memory',
    '-m',
    type=click.INT,
    default=4,
    help="NUMBER | Available memory (in GB) to use to load image data. "
    "A larger buffer means faster processing.")
def cli_reshuffle(img_path, ts_path, startdate, enddate, variables,
                  memory):
    """
    Convert SMOS L2 image data into a (5x5 degrees chunked) time series format
    following CF conventions (Indexed Ragged format).
    This format is preferred for performant location-based reading of SM data
    over the full period. To read the generated time series data, you can then
    use the `smos.smos_l2.interface.SmosL2Ts` or
    `pynetcf.time_series.GriddedNcIndexedRagged` class.

    \b
    Required Parameters
    -------------------
    IMG_PATH: string
        Path where previously downloaded SMOS images are stored. Use the
        `smos_l2 download` command to retrieve image data.
    TS_PATH: string
        Path where the newly created time series files should be stored.
    """
    # The docstring above is slightly different to the normal python one to
    # display it properly on the command line.
    print(f"Convert image data in {img_path} to time series in {ts_path}")

    variables = [str(v.strip()) for v in variables.split(',')]

    swath2ts(
        img_path,
        ts_path,
        variables=variables,
        startdate=startdate,
        enddate=enddate,
        memory=int(memory))

@click.command(
    "update_ts",
    context_settings={'show_default': True},
    short_help="Extend an existing time series record with "
    "available image data.")
@click.argument("img_path", type=click.Path(readable=True))
@click.argument("ts_path", type=click.Path(writable=True))
def cli_update_ts(img_path, ts_path):
    """
    Extend a locally existing SMOS L2 time series record by appending new data
    from the swath files. This will detect the time range of the time series
    data and compare it against the available image data.
    NOTE: Use the `smos_l2 reshuffle` program first do create a time series
    record to update with this function.

    \b
    Required Parameters
    -------------------
    IMG_PATH: string
        Path where previously downloaded SMOS files are stored.
    TS_PATH: string
        Path where the time series to update are stored
    """
    # The docstring above is slightly different to the normal python one to
    # display it properly on the command line.

    print(f"Extend time series in {ts_path} with image data from {img_path}")
    extend_ts(img_path, ts_path)

@click.group(short_help="SMOS L2 Command Line Programs.",
             name="smos_l2")
def smos_l2():
    pass

smos_l2.add_command(cli_download)
smos_l2.add_command(cli_update_img)
smos_l2.add_command(cli_reshuffle)
smos_l2.add_command(cli_update_ts)



import tempfile

import os
from smos.misc import get_first_last_day_images
import datetime

def test_first_last_date():
    rootf = os.path.join(os.path.join(os.path.dirname(__file__), 'smos-test-data'))

    with tempfile.TemporaryDirectory() as emptydir:
        s, e = get_first_last_day_images(emptydir)
        assert s is None
        assert e is None


    s, e = get_first_last_day_images(os.path.join(rootf, 'L2_SMOS'))
    assert s == datetime.date(2022,1,1)
    assert e == datetime.date(2022,1,3)

    s, e = get_first_last_day_images(os.path.join(rootf, 'L3_SMOS_IC', 'ASC'))
    assert s == datetime.date(2018,1,1)
    assert e == datetime.date(2018,1,3)

    s, e = get_first_last_day_images(os.path.join(rootf, 'L4_SMOS_RZSM', 'OPER'))
    assert s == datetime.date(2020,1,31)
    assert e == datetime.date(2020,1,31)


# -*- coding: utf-8 -*-
# The MIT License (MIT)
#
# Copyright (c) 2021,TU Wien
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import numpy as np
import numpy.testing as nptest
import tempfile
from smos.smos_l4.reshuffle_l4 import main
import glob
from smos.interface import SMOSTs


def test_SMOS_L4_reshuffle_global():
    inpath = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                          "smos-test-data", "L4_SMOS_RZSM", "SCIE")

    with tempfile.TemporaryDirectory() as ts_path:
        startdate = '2018-01-01'  # 1 day is missing, should be filled with nans
        enddate = '2018-01-03'
        parameters = ["--parameters", "RZSM", "QUAL"]

        args = [inpath, ts_path, startdate, enddate] + \
               parameters + ['--only_good', False]

        main(args)
        assert len(glob.glob(os.path.join(ts_path, "*.nc"))) == 2449
        ds = SMOSTs(ts_path, ioclass_kws={'read_bulk': True}, drop_missing=False)
        ts = ds.read(150.625, -32.125)  # this is the same point as in image test
        assert ts['QUAL'].dtype == float # because we dont drop missing
        sm_values_should = np.array([0.136741, 0.136160, np.nan], dtype=np.float32)
        nptest.assert_allclose(ts['RZSM'].values, sm_values_should, 4)
        ds.close()


def test_SMOS_L4_reshuffle_subset():
    inpath = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                          "smos-test-data", "L4_SMOS_RZSM", "SCIE")
    with tempfile.TemporaryDirectory() as ts_path:
        startdate = '2018-01-01'  # 1 day is missing, should be filled with nans
        enddate = '2018-01-03'
        bbox = ['-11', '34', '43', '71']
        args = [inpath, ts_path, startdate, enddate] + \
               ['--only_good', 'False'] + ['--bbox', *bbox]

        main(args)
        assert len(glob.glob(os.path.join(ts_path, "*.nc"))) == 109
        ds = SMOSTs(ts_path, drop_missing=False, ioclass_kws={'read_bulk': True})

        ts = ds.read(20.36023, 47.682177)
        timestamp0 = ts.index[0]
        nptest.assert_almost_equal(ts.loc[timestamp0, 'RZSM'], 0.248682, 4)
        assert ts['QUAL'].dtype == float
        assert ts['RZSM'].dtype == float
        ds.close()
        ds = SMOSTs(ts_path, drop_missing=False, ioclass_kws={'read_bulk': True})
        ts = ds.read(-61.08069, -12.55398)
        assert np.isnan(ts.loc['2018-01-01', 'RZSM'])
        assert np.isnan(ts.loc['2018-01-01', 'QUAL'])
        ds.close()


import os
from tempfile import TemporaryDirectory
from smos.smos_l2.download import SmosDissEoFtp
from smos.misc import read_summary_yml

def test_download_l2():
    with TemporaryDirectory() as tempdir:
        ftp = SmosDissEoFtp(local_root=tempdir, username='asd', password='asd',
                            skip_lftp_verify=True)
        c = ftp.sync(2022, 1, 1, opts='-e --testflag 1 2 3', dry_run=True)
        assert c == f'mirror -c -e --testflag 1 2 3 /SMOS/L2SM/MIR_SMUDP2_nc/2022/01/01 {os.path.join(tempdir, "2022", "01", "01")} −−no−perms'



def test_download_l2_period():
    with TemporaryDirectory() as tempdir:
        ftp = SmosDissEoFtp(local_root=tempdir, username='asd', password='asd',
                            skip_lftp_verify=True)
        cmds = ftp.sync_period('2022-01-01', '2022-01-03', dry_run=True)

        for d in [1, 2, 3]:
            assert cmds[d-1] == f'mirror -c /SMOS/L2SM/MIR_SMUDP2_nc/2022/01/0{d} {os.path.join(tempdir, "2022", "01", f"0{d}")} −−no−perms'

        props = read_summary_yml(tempdir)
        assert props['first_day'] is None
        assert props['last_day'] is None
        assert props['last_update'] is not None

if __name__ == '__main__':
    test_download_l2_period()